"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var FormBuilderDraggable_1 = require("./FormBuilderDraggable");
var FormBuilderEditable_1 = require("./FormBuilderEditable");
var FormBuilderDroppable_1 = require("./FormBuilderDroppable");
var FormBuilderContext_1 = require("./FormBuilderContext");
// FormBuilder expects a list of field definitions and will wrap each field definition
// in utility components for editing, dragging, and dropping. The FormBuilder uses
// a registry to determine which class is responsible for rendering the field type.
var FormBuilder = (function (_super) {
    __extends(FormBuilder, _super);
    function FormBuilder(props) {
        var _this = _super.call(this, props) || this;
        _this.renderField = _this.renderField.bind(_this);
        _this.onDrop = _this.onDrop.bind(_this);
        _this.onEditField = _this.onEditField.bind(_this);
        _this.onDeleteField = _this.onDeleteField.bind(_this);
        _this.onFieldChanged = _this.onFieldChanged.bind(_this);
        _this.state = {
            editingField: null
        };
        return _this;
    }
    // onEditField is called when the user wants to edit a field.
    // If the `onBeforeEditField` hook is provided, it will be used
    // to determine if the field can be edited.
    FormBuilder.prototype.onEditField = function (field) {
        var hook = this.props.onBeforeEditField;
        if (hook && !hook(field)) {
            return;
        }
        this.setState({ editingField: field });
        this.props.onFieldEditing(field, { fields: this.props.fields }, function (field) {
            var editingIndex = this.props.fields.indexOf(this.state.editingField);
            var fields = this.props.fields.slice();
            fields[editingIndex] = field;
            this.setState({ editingField: field });
            this.props.onChange(fields);
        }.bind(this));
    };
    // onDeleteField is called when the user wants to delete a field.
    // If the `onBeforeDeleteField` hook is provided, it will be used
    // to determine if the field can be deleted.
    FormBuilder.prototype.onDeleteField = function (index) {
        var fields = this.props.fields.concat([]);
        var field = this.props.fields[index];
        var hook = this.props.onBeforeDeleteField;
        if (hook && !hook(field)) {
            return;
        }
        if (field === this.state.editingField) {
            this.setState({ editingField: null });
        }
        fields.splice(index, 1);
        this.props.onChange(fields);
    };
    // onDrop is called whenever a field is dropped on a <Droppable> component.
    // The field being dropped can come from one of two places:
    //      1. The FieldSelector component
    //      2. A <Draggable/> component being reordered.
    // target and source both contain the index and the IField of the field
    // being dragged, and the field being dropped on.
    // If a field is being dragged from <FieldSelector> then it will not have an index
    // and should simply be inserted.
    // If the source field comes from a <Draggable> component then it must
    // go through removal and insertion. Removal and insertion depend
    // on the source index to keep the integrity of the target index.
    FormBuilder.prototype.onDrop = function (target, source, didDrop) {
        if (didDrop) {
            return;
        }
        if (target.index === source.index) {
            return;
        }
        var sourceField = source.field;
        if (source.index === null) {
            // NOTE: If source is from the FieldSelector, we should create a clone field.
            sourceField = JSON.parse(JSON.stringify(sourceField));
            var hook = this.props.onBeforeAddField;
            if (hook && !hook(sourceField)) {
                return;
            }
        }
        else if (this.props.fields.indexOf(target.field) !== target.index &&
            this.props.fields.indexOf(source.field) !== source.index) {
            // NOTE: For re-ordering, make sure the target & source are in the same level.
            return;
        }
        var fields = this.props.fields.concat([]);
        if (source.index === null) {
            fields.splice(target.index, 0, sourceField);
        }
        else if (source.index < target.index) {
            fields.splice(target.index, 0, sourceField);
            fields.splice(source.index, 1);
        }
        else {
            fields.splice(source.index, 1);
            fields.splice(target.index, 0, sourceField);
        }
        this.props.onChange(fields);
    };
    FormBuilder.prototype.onFieldChanged = function (field, index) {
        var fields = this.props.fields.slice();
        fields[index] = field;
        this.props.onChange(fields);
    };
    // renderField takes the field.type to be rendered and looks up the
    // appropriate component class in the registry that can render the component.
    // The rendered component is passed the field as a prop.
    FormBuilder.prototype.renderField = function (field, index) {
        var fieldDef = this.props.registry[field.type];
        if (!fieldDef || !fieldDef.builder) {
            console.warn('Field defintion is not registered: ' + field.type);
            return;
        }
        var fieldBuilderProps = {
            field: field,
            index: index,
            registry: this.props.registry,
            editButton: this.props.editButton,
            deleteButton: this.props.deleteButton,
            onFieldEditing: this.props.onFieldEditing,
            onChange: this.onFieldChanged,
            onBeforeAddField: this.props.onBeforeAddField,
        };
        var component = React.createElement(fieldDef.builder, fieldBuilderProps);
        var isEditing = (field === this.state.editingField);
        return (React.createElement("div", { className: 'form-builder-field', key: index },
            React.createElement(FormBuilderDroppable_1.FormBuilderDroppable, { index: index, onDrop: this.onDrop, field: field },
                React.createElement(FormBuilderEditable_1.FormBuilderEditable, { deleteButton: this.props.deleteButton, editButton: this.props.editButton, onEdit: this.onEditField, onDelete: this.onDeleteField, index: index, field: field, isEditing: isEditing },
                    React.createElement(FormBuilderDraggable_1.FormBuilderDraggable, { index: index, field: field }, component)))));
    };
    // render displays a list of fields that have been wrapped in components for
    // providing editing, dragging, and dropping functionality. At the end of the list
    // is a <Droppable> field.
    FormBuilder.prototype.render = function () {
        return (React.createElement(FormBuilderContext_1.FormBuilderContext, null,
            React.createElement("div", { className: 'form-builder' },
                this.props.fields.map(this.renderField),
                React.createElement(FormBuilderDroppable_1.FormBuilderDroppable, { index: this.props.fields.length, field: null, onDrop: this.onDrop },
                    React.createElement("div", { style: { padding: 25 } })))));
    };
    return FormBuilder;
}(React.Component));
exports.FormBuilder = FormBuilder;
